@@ -13,6 +13,7 @@
 
 #include <KFileItem>
 #include <KIO/ApplicationLauncherJob>
+#include <KIO/StatJob>
 #include <KLocalizedString>
 #include <KPluginFactory>
 
@@ -50,25 +51,41 @@ QList<QAction *> CompressFileItemAction::actions(const KFileItemListProperties &
     }
 
     QList<QAction *> actions;
+    QList<QAction *> actionsToBeDisabledInReadOnlyDir;
     const QIcon icon = QIcon::fromTheme(QStringLiteral("archive-insert"));
 
     QMenu *compressMenu = new QMenu(parentWidget);
 
     compressMenu->addAction(createAction(icon, parentWidget, urlList, QStringLiteral("tar.gz")));
+    actionsToBeDisabledInReadOnlyDir << compressMenu->actions().last();
 
     const QMimeType zipMime = QMimeDatabase().mimeTypeForName(QStringLiteral("application/zip"));
     // Don't offer zip compression if no zip plugin is available.
     if (!m_pluginManager->preferredWritePluginsFor(zipMime).isEmpty()) {
         compressMenu->addAction(createAction(icon, parentWidget, urlList, QStringLiteral("zip")));
+        actionsToBeDisabledInReadOnlyDir << compressMenu->actions().last();
     }
 
     compressMenu->addAction(createAction(icon, parentWidget, urlList, QString()));
 
     QAction *compressMenuAction = new QAction(i18nc("@action:inmenu Compress submenu in Dolphin context menu", "Compress"), parentWidget);
     compressMenuAction->setMenu(compressMenu);
-    compressMenuAction->setEnabled(fileItemInfos.isLocal() && fileItemInfos.supportsWriting() && !m_pluginManager->availableWritePlugins().isEmpty());
+    compressMenuAction->setEnabled(fileItemInfos.isLocal() && !m_pluginManager->availableWritePlugins().isEmpty());
     compressMenuAction->setIcon(icon);
 
+    if (compressMenuAction->isEnabled()) {
+        const KFileItemList items = fileItemInfos.items();
+        const KFileItem &first = items.first();
+        auto *job = KIO::stat(first.url().adjusted(QUrl::RemoveFilename | QUrl::StripTrailingSlash));
+        connect(job, &KJob::result, compressMenu, [actionsToBeDisabledInReadOnlyDir, job]() {
+            if (!job->error() && !KFileItem(job->statResult(), job->url()).isWritable()) {
+                for (auto action : actionsToBeDisabledInReadOnlyDir) {
+                    action->setEnabled(false);
+                }
+            }
+        });
+    }
+
     actions << compressMenuAction;
     return actions;
 }
@@ -84,7 +101,8 @@ QAction *CompressFileItemAction::createAction(const QIcon &icon, QWidget *parent
             fileName = fileName.left(10) + QStringLiteral("…") + fileName.right(10);
         }
 
-        name = i18nc("@action:inmenu Part of Compress submenu in Dolphin context menu, %1 filename", "Here as \"%1\"", fileName);
+        fileName.replace(QStringLiteral("&"), QStringLiteral("&&"));
+        name = i18nc("@action:inmenu Part of Compress submenu in Dolphin context menu, %1 filename", "Compress to \"%1\"", fileName);
     }
 
     QAction *action = new QAction(icon, name, parent);
